@@ -17,9 +17,9 @@ the source of this lib to deal with the FFI types and unsafe code.
 
 CSPICE | **rust-spice** | Description
 -------|--------------|------------
-[bodc2n_c][bodc2n_c link] | *TODO*
+[bodc2n_c][bodc2n_c link] | [`neat::bodc2n`] | Body ID code to name translation
 [bodfnd_c][bodfnd_c link] | *TODO*
-[bodn2c_c][bodn2c_c link] | *TODO*
+[bodn2c_c][bodn2c_c link] | [`raw::bodn2c`] | Body name to ID code translation
 [bodvrd_c][bodvrd_c link] | *TODO*
 [ckcov_c][ckcov_c link] | *TODO*
 [ckgp_c][ckgp_c link] | *TODO*
@@ -30,19 +30,25 @@ CSPICE | **rust-spice** | Description
 [dlabfs_c][dlabfs_c link] | [`raw::dlabfs`] | DLA, begin forward search
 [dskgd_c][dskgd_c link] | [`raw::dskgd`] | DSK, return DSK segment descriptor
 [dskn02_c][dskn02_c link] | [`raw::dskn02`] | DSK, type 2, compute normal vector for plate
-[dskobj_c][dskobj_c link] | *TODO*
-[dskp02_c][dskp02_c link] | *TODO*
+[dskobj_c][dskobj_c link] | [`raw::dskobj`] | DSK, get object IDs
+[dskp02_c][dskp02_c link] | [`neat::dskp02`] | DSK, fetch type 2 plate data
 [dsksrf_c][dsksrf_c link] | *TODO*
-[dskv02_c][dskv02_c link] | *TODO*
+[dskv02_c][dskv02_c link] | [`neat::dskv02`] | DSK, fetch type 2 vertex data
 [dskx02_c][dskx02_c link] | [`raw::dskx02`] | DSK, ray-surface intercept, type 2
-[dskz02_c][dskz02_c link] | *TODO*
+[dskz02_c][dskz02_c link] | [`raw::dskz02`] | DSK, fetch type 2 model size parameters
 [furnsh_c][furnsh_c link] | [`raw::furnsh`] | Furnish a program with SPICE kernels
 [gcpool_c][gcpool_c link] | *TODO*
 [gdpool_c][gdpool_c link] | *TODO*
+[georec_c][georec_c link] | [`raw::georec`] |  Geodetic to rectangular coordinates
 [getfov_c][getfov_c link] | *TODO*
 [gipool_c][gipool_c link] | *TODO*
+[illumf_c][illumf_c link] | [`raw::illumf`] | Illumination angles, general source, return flags
+[kclear_c][kclear_c link] | [`raw::kclear`] | Keeper clear
+[kdata_c][kdata_c link] | [`neat::kdata`] | Kernel Data
+[ktotal_c][ktotal_c link] | [`raw::ktotal`] | Kernel Totals
 [latrec_c][latrec_c link] | [`raw::latrec`] | Latitudinal to rectangular coordinates
 [latsrf_c][latsrf_c link] | *TODO*
+[mxv_c][mxv_c link] | [`raw::mxv`] |  Matrix times vector, 3x3
 [occult_c][occult_c link] | [`raw::occult`] | Find occultation type at time
 [pckcov_c][pckcov_c link] | *TODO*
 [pxform_c][pxform_c link] | [`raw::pxform`] | Position Transformation Matrix
@@ -58,7 +64,7 @@ CSPICE | **rust-spice** | Description
 [spkcpt_c][spkcpt_c link] | *TODO*
 [spkcvo_c][spkcvo_c link] | *TODO*
 [spkcvt_c][spkcvt_c link] | *TODO*
-[spkezr_c][spkezr_c link] | *TODO*
+[spkezr_c][spkezr_c link] | [`raw::spkezr`] | S/P Kernel, easier reader
 [spkobj_c][spkobj_c link] | *TODO*
 [spkpos_c][spkpos_c link] | [`raw::spkpos`] | S/P Kernel, position
 [srfc2s_c][srfc2s_c link] | *TODO*
@@ -68,10 +74,14 @@ CSPICE | **rust-spice** | Description
 [srfscc_c][srfscc_c link] | *TODO*
 [str2et_c][str2et_c link] | [`raw::str2et`] | String to ET
 [sxform_c][sxform_c link] | *TODO*
+[radrec_c][radrec_c link] | [`raw::radrec`] |  RA and DEC to rectangular coordinates
 [recrad_c][recrad_c link] | [`raw::recrad`] | Rectangular coordinates to RA and DEC
 [timout_c][timout_c link] | [`neat::timout`] | Time Output
 [unload_c][unload_c link] | [`raw::unload`] | Unload a kernel
+[vcrss_c][vcrss_c link] | [`raw::vcrss`] | Vector cross product, 3 dimensions
+[vdot_c][vdot_c link] | [`raw::vdot`] |  Vector dot product, 3 dimensions
 [vsep_c][vsep_c link] | [`raw::vsep`] | Angular separation of vectors, 3 dimensions
+[xpose_c][xpose_c link] | [`raw::xpose`] | Transpose a matrix, 3x3
 
 [bodc2n_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/bodc2n_c.html
 [bodfnd_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/bodfnd_c.html
@@ -96,9 +106,15 @@ CSPICE | **rust-spice** | Description
 [gcpool_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/gcpool_c.html
 [gdpool_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/gdpool_c.html
 [getfov_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/getfov_c.html
+[georec_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/georec_c.html
 [gipool_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/gipool_c.html
+[illumf_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/illumf_c.html
+[kclear_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/kclear_c.html
+[kdata_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/kdata_c.html
+[ktotal_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/ktotal_c.html
 [latrec_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/latrec_c.html
 [latsrf_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/latsrf_c.html
+[mxv_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/mxv_c.html
 [occult_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/occult_c.html
 [pxform_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/pxform_c.html
 [pckcov_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/pckcov_c.html
@@ -125,19 +141,24 @@ CSPICE | **rust-spice** | Description
 [srfscc_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/srfscc_c.html
 [str2et_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/str2et_c.html
 [sxform_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/sxform_c.html
+[radrec_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/radrec_c.html
 [recrad_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/recrad_c.html
 [timout_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/timout_c.html
 [unload_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/unload_c.html
+[vcrss_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/vcrss_c.html
+[vdot_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/vdot_c.html
 [vsep_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/vsep_c.html
+[xpose_c link]: https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/xpose_c.html
 */
 
 pub mod neat;
 pub mod raw;
 
-pub use self::neat::timout;
+pub use self::neat::{bodc2n, dskp02, dskv02, kdata, timout};
 pub use self::raw::{
-    dascls, dasopr, dlabfs, dskgd, dskn02, dskx02, furnsh, latrec, pxform, pxfrm2, recrad, spkpos,
-    str2et, unload, vsep,
+    bodn2c, dascls, dasopr, dlabfs, dskgd, dskn02, dskobj, dskx02, dskz02, furnsh, georec, illumf,
+    kclear, ktotal, latrec, mxv, pxform, pxfrm2, radrec, recrad, spkezr, spkpos, str2et, unload,
+    vcrss, vdot, vsep, xpose, DLADSC, DSKDSC,
 };
 
 /**
@@ -150,6 +171,31 @@ Size of the default date format.
 */
 pub const TIME_FORMAT_SIZE: usize = TIME_FORMAT.len();
 
+/**
+Maximum size of string outputs.
+*/
+pub const MAX_LEN_OUT: usize = 256;
+
+/**
+Allocate for a given type and number of elements.
+*/
+#[macro_export]
+macro_rules! malloc {
+    ($a:ty, $n:expr) => {
+        unsafe { libc::malloc(std::mem::size_of::<$a>() * $n as usize) as *mut $a }
+    };
+}
+
+/**
+Allocate [`*mut i8`][`std::os::raw::c_char`] to be sent as a pointer to a string.
+*/
+#[macro_export]
+macro_rules! mallocstr {
+    ($s:expr) => {
+        crate::malloc!(i8, $s + 1);
+    };
+}
+
 /**
 Convert [`String`] to [`*mut i8`][`std::os::raw::c_char`].
 */
@@ -172,3 +218,43 @@ macro_rules! fcstr {
         unsafe { std::ffi::CStr::from_ptr($s).to_str().unwrap().to_string() }
     }};
 }
+
+/**
+Pointer to expression.
+*/
+#[macro_export]
+macro_rules! mptr {
+    ($e:expr) => {
+        $e.as_mut_ptr()
+    };
+}
+
+/**
+Allocate a scalar to be sent as a pointer.
+*/
+#[macro_export]
+macro_rules! init_scalar {
+    () => {
+        std::mem::MaybeUninit::uninit();
+    };
+}
+
+/**
+Retrieve a value from a pointer to a scalar created using [`init_scalar`].
+*/
+#[macro_export]
+macro_rules! get_scalar {
+    ($e:expr) => {
+        $e.assume_init()
+    };
+}
+
+/**
+Get a vector of array from the pointer to the array.
+*/
+#[macro_export]
+macro_rules! get_varr {
+    ($e:expr, $n:expr) => {
+        std::slice::from_raw_parts($e, $n as usize).to_vec();
+    };
+}
